@@ -50,6 +50,7 @@ pub struct ModuleInfo<'a> {
     pub function_map: Vec<u32>,
     pub global_types: Vec<PrimitiveTypeInfo>,
     pub table_elem_types: Vec<PrimitiveTypeInfo>,
+    pub memory_types: Vec<wasmparser::MemoryType>,
 
     // raw_sections
     pub raw_sections: Vec<RawSection<'a>>,
@@ -104,32 +105,30 @@ impl<'a> ModuleInfo<'a> {
                     for _ in 0..reader.get_count() {
                         let ty = reader.read()?;
                         match ty.ty {
-                            wasmparser::ImportSectionEntryType::Function(ty) => {
+                            wasmparser::TypeRef::Func(ty) => {
                                 // Save imported functions
                                 info.function_map.push(ty);
                                 info.imported_functions_count += 1;
                             }
-                            wasmparser::ImportSectionEntryType::Global(ty) => {
+                            wasmparser::TypeRef::Global(ty) => {
                                 let ty = PrimitiveTypeInfo::try_from(ty.content_type).unwrap();
                                 info.global_types.push(ty);
                                 info.imported_globals_count += 1;
                             }
-                            wasmparser::ImportSectionEntryType::Memory(_ty) => {
+                            wasmparser::TypeRef::Memory(ty) => {
                                 info.memory_count += 1;
                                 info.imported_memories_count += 1;
+                                info.memory_types.push(ty);
                             }
-                            wasmparser::ImportSectionEntryType::Table(ty) => {
+                            wasmparser::TypeRef::Table(ty) => {
                                 info.table_count += 1;
                                 info.imported_tables_count += 1;
                                 info.table_elem_types.push(ty.element_type.into());
                             }
-                            wasmparser::ImportSectionEntryType::Tag(_ty) => {
+                            wasmparser::TypeRef::Tag(_ty) => {
                                 info.tag_count += 1;
                                 info.imported_tags_count += 1;
                             }
-                            _ => {
-                                // Do nothing
-                            }
                         }
                     }
                 }
@@ -154,10 +153,15 @@ impl<'a> ModuleInfo<'a> {
                         info.table_elem_types.push(ty);
                     }
                 }
-                Payload::MemorySection(reader) => {
+                Payload::MemorySection(mut reader) => {
                     info.memories = Some(info.raw_sections.len());
                     info.memory_count += reader.get_count();
                     info.section(SectionId::Memory.into(), reader.range(), input_wasm);
+
+                    for _ in 0..reader.get_count() {
+                        let ty = reader.read()?;
+                        info.memory_types.push(ty);
+                    }
                 }
                 Payload::GlobalSection(mut reader) => {
                     info.globals = Some(info.raw_sections.len());
@@ -165,7 +169,7 @@ impl<'a> ModuleInfo<'a> {
 
                     for _ in 0..reader.get_count() {
                         let ty = reader.read()?;
-                        // We only need the type of the global, not necesarily if is mutable or not
+                        // We only need the type of the global, not necessarily if is mutable or not
                         let ty = PrimitiveTypeInfo::try_from(ty.ty.content_type).unwrap();
                         info.global_types.push(ty);
                     }
@@ -176,7 +180,7 @@ impl<'a> ModuleInfo<'a> {
 
                     for _ in 0..reader.get_count() {
                         let entry = reader.read()?;
-                        info.export_names.insert(entry.field.into());
+                        info.export_names.insert(entry.name.into());
                     }
 
                     info.section(SectionId::Export.into(), reader.range(), input_wasm);
@@ -204,9 +208,6 @@ impl<'a> ModuleInfo<'a> {
                 } => {
                     info.section(SectionId::Custom.into(), range, input_wasm);
                 }
-                Payload::AliasSection(reader) => {
-                    info.section(SectionId::Alias.into(), reader.range(), input_wasm);
-                }
                 Payload::UnknownSection {
                     id,
                     contents: _,
@@ -219,7 +220,7 @@ impl<'a> ModuleInfo<'a> {
                     info.section(SectionId::DataCount.into(), range, input_wasm);
                 }
                 Payload::Version { .. } => {}
-                Payload::End => {
+                Payload::End(_) => {
                     break;
                 }
                 _ => todo!("{:?} not implemented", payload),
@@ -253,6 +254,11 @@ impl<'a> ModuleInfo<'a> {
         });
     }
 
+    pub fn get_type_section(&self) -> Option<RawSection<'a>> {
+        let idx = self.types?;
+        Some(self.raw_sections[idx])
+    }
+
     pub fn get_code_section(&self) -> RawSection<'a> {
         self.raw_sections[self.code.unwrap()]
     }
@@ -287,6 +293,26 @@ impl<'a> ModuleInfo<'a> {
         self.raw_sections[self.globals.unwrap()]
     }
 
+    /// Insert a new section as the `i`th section in the Wasm module.
+    pub fn insert_section(
+        &self,
+        i: usize,
+        new_section: &impl wasm_encoder::Section,
+    ) -> wasm_encoder::Module {
+        let mut module = wasm_encoder::Module::new();
+        self.raw_sections.iter().enumerate().for_each(|(j, s)| {
+            if i == j {
+                module.section(new_section);
+            }
+            module.section(s);
+        });
+        if self.raw_sections.len() == i {
+            module.section(new_section);
+        }
+        module
+    }
+
+    /// Replace the `i`th section in this module with the given new section.
     pub fn replace_section(
         &self,
         i: usize,
@@ -312,9 +338,9 @@ impl<'a> ModuleInfo<'a> {
     /// * `section_writer` this callback should write the custom section and
     ///   returns true if it was successful, if false is returned then the
     ///   default section will be written to the module
-    pub fn replace_multiple_sections<P>(&self, section_writer: P) -> wasm_encoder::Module
+    pub fn replace_multiple_sections<P>(&self, mut section_writer: P) -> wasm_encoder::Module
     where
-        P: Fn(usize, u8, &mut wasm_encoder::Module) -> bool,
+        P: FnMut(usize, u8, &mut wasm_encoder::Module) -> bool,
     {
         let mut module = wasm_encoder::Module::new();
         self.raw_sections.iter().enumerate().for_each(|(j, s)| {
@@ -362,6 +388,10 @@ impl<'a> ModuleInfo<'a> {
         self.imported_globals_count
     }
 
+    pub fn num_local_globals(&self) -> u32 {
+        self.global_types.len() as u32 - self.imported_globals_count
+    }
+
     pub fn num_tags(&self) -> u32 {
         self.tag_count
     }
