@@ -17,10 +17,12 @@ mod mutators;
 pub use error::*;
 
 use crate::mutators::{
-    codemotion::CodemotionMutator, custom::RemoveCustomSection,
+    add_function::AddFunctionMutator, add_type::AddTypeMutator, codemotion::CodemotionMutator,
     function_body_unreachable::FunctionBodyUnreachable, modify_data::ModifyDataMutator,
-    peephole::PeepholeMutator, remove_export::RemoveExportMutator, remove_item::RemoveItemMutator,
-    rename_export::RenameExportMutator, snip_function::SnipMutator, Item,
+    modify_init_exprs::InitExpressionMutator, peephole::PeepholeMutator,
+    remove_export::RemoveExportMutator, remove_item::RemoveItemMutator,
+    remove_section::RemoveSection, rename_export::RenameExportMutator, snip_function::SnipMutator,
+    Item,
 };
 use info::ModuleInfo;
 use mutators::Mutator;
@@ -159,8 +161,17 @@ pub struct WasmMutate<'wasm> {
     preserve_semantics: bool,
 
     /// Fuel to control the time of the mutation.
-    #[cfg_attr(feature = "clap", clap(skip = Cell::new(u64::MAX)))]
+    #[cfg_attr(
+        feature = "clap",
+        clap(
+            short,
+            long,
+            default_value = "18446744073709551615", // u64::MAX
+            parse(try_from_str = parse_fuel),
+        )
+    )]
     fuel: Cell<u64>,
+
     /// Only perform size-reducing transformations on the Wasm module. This
     /// allows `wasm-mutate` to be used as a test case reducer.
     #[cfg_attr(feature = "clap", clap(long))]
@@ -178,6 +189,11 @@ pub struct WasmMutate<'wasm> {
     info: Option<ModuleInfo<'wasm>>,
 }
 
+#[cfg(feature = "clap")]
+fn parse_fuel(s: &str) -> Result<Cell<u64>, String> {
+    s.parse::<u64>().map(Cell::new).map_err(|s| s.to_string())
+}
+
 impl Default for WasmMutate<'_> {
     fn default() -> Self {
         let seed = 3;
@@ -278,7 +294,16 @@ impl<'wasm> WasmMutate<'wasm> {
                 SnipMutator,
                 CodemotionMutator,
                 FunctionBodyUnreachable,
-                RemoveCustomSection,
+                AddTypeMutator {
+                    max_params: 20,
+                    max_results: 20,
+                },
+                AddFunctionMutator,
+                RemoveSection::Custom,
+                RemoveSection::Empty,
+                InitExpressionMutator::Global,
+                InitExpressionMutator::ElementOffset,
+                InitExpressionMutator::ElementFunc,
                 RemoveItemMutator(Item::Function),
                 RemoveItemMutator(Item::Global),
                 RemoveItemMutator(Item::Memory),
@@ -352,9 +377,14 @@ impl<'wasm> WasmMutate<'wasm> {
 }
 
 #[cfg(test)]
-pub(crate) fn validate(validator: &mut wasmparser::Validator, bytes: &[u8]) {
+pub(crate) fn validate(bytes: &[u8]) {
+    let mut validator = wasmparser::Validator::new_with_features(wasmparser::WasmFeatures {
+        memory64: true,
+        multi_memory: true,
+        ..Default::default()
+    });
     let err = match validator.validate_all(bytes) {
-        Ok(()) => return,
+        Ok(_) => return,
         Err(e) => e,
     };
     drop(std::fs::write("test.wasm", &bytes));
@@ -362,5 +392,5 @@ pub(crate) fn validate(validator: &mut wasmparser::Validator, bytes: &[u8]) {
         drop(std::fs::write("test.wat", &text));
     }
 
-    panic!("wasm failed to validate {:?}", err);
+    panic!("wasm failed to validate: {} (written to test.wasm)", err);
 }
