@@ -20,13 +20,16 @@
 //! functions is processed in order to construct an equivalent piece of code.
 //!
 
+pub mod add_function;
+pub mod add_type;
 pub mod codemotion;
-pub mod custom;
 pub mod function_body_unreachable;
 pub mod modify_data;
+pub mod modify_init_exprs;
 pub mod peephole;
 pub mod remove_export;
 pub mod remove_item;
+pub mod remove_section;
 pub mod rename_export;
 pub mod snip_function;
 pub mod start;
@@ -117,7 +120,6 @@ impl WasmMutate<'_> {
         T: Mutator + Clone,
     {
         use crate::ErrorKind;
-        use wasmparser::WasmFeatures;
 
         drop(env_logger::try_init());
 
@@ -150,12 +152,7 @@ impl WasmMutate<'_> {
 
             let mutation_bytes = mutation.finish();
 
-            let mut validator = wasmparser::Validator::new();
-            validator.wasm_features(WasmFeatures {
-                multi_memory: true,
-                ..WasmFeatures::default()
-            });
-            crate::validate(&mut validator, &mutation_bytes);
+            crate::validate(&mutation_bytes);
 
             // If it fails, it is probably an invalid
             // reformatting expected
