@@ -1,16 +1,15 @@
 //! DFG extractor for Wasm functions. It converts Wasm operators to the [Lang]
 //! intermediate representation.
-use std::collections::HashMap;
-
-use egg::{Id, Language, RecExpr};
-use wasmparser::{Operator, Range};
-
-use crate::mutators::peephole::{Lang, MemArg, MemoryCopy, MemoryInit, TableCopy, TableInit};
-use crate::ModuleInfo;
-
-use crate::mutators::OperatorAndByteOffset;
 
 use super::eggsy::encoder::rebuild::build_expr;
+use crate::mutators::peephole::{
+    Lang, MemArg, MemArgLane, MemoryCopy, MemoryInit, RefType, TableCopy, TableInit,
+};
+use crate::mutators::OperatorAndByteOffset;
+use crate::{ModuleInfo, WasmMutate};
+use egg::{Id, Language, RecExpr};
+use std::collections::HashMap;
+use wasmparser::{Operator, Range};
 
 /// It executes a minimal symbolic evaluation of the stack to detect operands
 /// location in the code for certain operators
@@ -28,6 +27,7 @@ pub struct DFGBuilder {
     dfg_map: Vec<StackEntry>,
     operator_index_to_entry_index: HashMap<usize, usize>,
     parents: Vec<Option<usize>>,
+    preserve_semantics: bool,
 }
 
 /// Basic block of a Wasm's function defined as a range of operators in the Wasm
@@ -90,6 +90,7 @@ impl MiniDFG {
             .or(Some(false))
             .unwrap()
     }
+
     /// Return true if the coloring of the children subtrees is the same as the root
     /// Notice that this value can be calcuated when the tree is built
     pub fn is_subtree_consistent_from_root(&self) -> bool {
@@ -102,6 +103,7 @@ impl MiniDFG {
     pub fn pretty_print_default(&self) -> String {
         self.pretty_print(&|entry: &StackEntry| format!("{:?}", entry.operator))
     }
+
     /// Pretty prints the DFG forest in a tree structure
     /// It receives a custom function to format how the entry information will be written
     /// For an example, see the implementation of the `pretty_print_default` method.
@@ -200,13 +202,14 @@ const UNDEF_COLOR: u32 = u32::MAX;
 
 impl<'a> DFGBuilder {
     /// Returns a new DFG builder
-    pub fn new() -> Self {
+    pub fn new(config: &WasmMutate) -> Self {
         DFGBuilder {
             color: 0,
             stack: Vec::new(),
             dfg_map: Vec::new(),
             operator_index_to_entry_index: HashMap::new(),
             parents: Vec::new(),
+            preserve_semantics: config.preserve_semantics,
         }
     }
 
@@ -339,8 +342,14 @@ impl<'a> DFGBuilder {
     }
 
     fn new_color(&mut self) {
-        self.color += 1;
-        assert!(self.color < UNDEF_COLOR);
+        // If we're not in semantics preservation mode then there's no need to
+        // ever switch colors because the color here is only used to ensure that
+        // we don't break edges between effectful instructions, which is all
+        // about preserving semantics.
+        if self.preserve_semantics {
+            self.color += 1;
+            assert!(self.color < UNDEF_COLOR);
+        }
     }
 
     /// This method should build lane dfg information
@@ -418,6 +427,9 @@ impl<'a> DFGBuilder {
                 Operator::F64Const { value } => {
                     self.push_node(Lang::F64(value.bits()), idx);
                 }
+                Operator::V128Const { value } => {
+                    self.push_node(Lang::V128(value.i128()), idx);
+                }
                 Operator::LocalSet { local_index } => {
                     let val = self.pop_operand(idx, true);
                     self.empty_node(Lang::LocalSet(*local_index, Id::from(val)), idx);
@@ -428,6 +440,10 @@ impl<'a> DFGBuilder {
                     self.new_color();
                 }
 
+                Operator::Nop => {
+                    self.empty_node(Lang::Nop, idx);
+                }
+
                 Operator::I32Store { memarg } => self.store(idx, memarg, Lang::I32Store),
                 Operator::I64Store { memarg } => self.store(idx, memarg, Lang::I64Store),
                 Operator::F32Store { memarg } => self.store(idx, memarg, Lang::F32Store),
@@ -589,6 +605,14 @@ impl<'a> DFGBuilder {
                 Operator::I64Rotl => self.binop(idx, Lang::I64RotL),
                 Operator::I64Rotr => self.binop(idx, Lang::I64RotR),
 
+                Operator::V128Not => self.unop(idx, Lang::V128Not),
+                Operator::V128And => self.binop(idx, Lang::V128And),
+                Operator::V128AndNot => self.binop(idx, Lang::V128AndNot),
+                Operator::V128Or => self.binop(idx, Lang::V128Or),
+                Operator::V128Xor => self.binop(idx, Lang::V128Xor),
+                Operator::V128AnyTrue => self.unop(idx, Lang::V128AnyTrue),
+                Operator::V128Bitselect => self.ternop(idx, Lang::V128Bitselect),
+
                 Operator::Drop => {
                     let arg = self.pop_operand(idx, false);
                     self.empty_node(Lang::Drop([Id::from(arg)]), idx);
@@ -735,11 +759,333 @@ impl<'a> DFGBuilder {
                 Operator::TableSet { table } => {
                     let arg1 = Id::from(self.pop_operand(idx, false));
                     let arg2 = Id::from(self.pop_operand(idx, false));
-                    self.push_node(Lang::TableSet(*table, [arg2, arg1]), idx);
+                    self.empty_node(Lang::TableSet(*table, [arg2, arg1]), idx);
                 }
 
-                _ => {
-                    // If the operator is not implemented, break the mutation of this Basic Block
+                Operator::RefNull {
+                    ty: wasmparser::Type::ExternRef,
+                } => {
+                    self.push_node(Lang::RefNull(RefType::Extern), idx);
+                }
+                Operator::RefNull {
+                    ty: wasmparser::Type::FuncRef,
+                } => {
+                    self.push_node(Lang::RefNull(RefType::Func), idx);
+                }
+                Operator::RefFunc { function_index } => {
+                    self.push_node(Lang::RefFunc(*function_index), idx);
+                }
+
+                Operator::RefIsNull => {
+                    let arg = Id::from(self.pop_operand(idx, false));
+                    self.push_node(Lang::RefIsNull(arg), idx);
+                }
+
+                Operator::V128Load { memarg } => self.load(idx, memarg, Lang::V128Load),
+                Operator::V128Load8x8S { memarg } => self.load(idx, memarg, Lang::V128Load8x8S),
+                Operator::V128Load8x8U { memarg } => self.load(idx, memarg, Lang::V128Load8x8U),
+                Operator::V128Load16x4S { memarg } => self.load(idx, memarg, Lang::V128Load16x4S),
+                Operator::V128Load16x4U { memarg } => self.load(idx, memarg, Lang::V128Load16x4U),
+                Operator::V128Load32x2S { memarg } => self.load(idx, memarg, Lang::V128Load32x2S),
+                Operator::V128Load32x2U { memarg } => self.load(idx, memarg, Lang::V128Load32x2U),
+                Operator::V128Load8Splat { memarg } => self.load(idx, memarg, Lang::V128Load8Splat),
+                Operator::V128Load16Splat { memarg } => {
+                    self.load(idx, memarg, Lang::V128Load16Splat)
+                }
+                Operator::V128Load32Splat { memarg } => {
+                    self.load(idx, memarg, Lang::V128Load32Splat)
+                }
+                Operator::V128Load64Splat { memarg } => {
+                    self.load(idx, memarg, Lang::V128Load64Splat)
+                }
+                Operator::V128Load32Zero { memarg } => self.load(idx, memarg, Lang::V128Load32Zero),
+                Operator::V128Load64Zero { memarg } => self.load(idx, memarg, Lang::V128Load64Zero),
+                Operator::V128Store { memarg } => self.store(idx, memarg, Lang::V128Store),
+                Operator::V128Load8Lane { memarg, lane } => {
+                    self.load_lane(idx, memarg, lane, Lang::V128Load8Lane)
+                }
+                Operator::V128Load16Lane { memarg, lane } => {
+                    self.load_lane(idx, memarg, lane, Lang::V128Load16Lane)
+                }
+                Operator::V128Load32Lane { memarg, lane } => {
+                    self.load_lane(idx, memarg, lane, Lang::V128Load32Lane)
+                }
+                Operator::V128Load64Lane { memarg, lane } => {
+                    self.load_lane(idx, memarg, lane, Lang::V128Load64Lane)
+                }
+                Operator::V128Store8Lane { memarg, lane } => {
+                    self.store_lane(idx, memarg, lane, Lang::V128Store8Lane)
+                }
+                Operator::V128Store16Lane { memarg, lane } => {
+                    self.store_lane(idx, memarg, lane, Lang::V128Store16Lane)
+                }
+                Operator::V128Store32Lane { memarg, lane } => {
+                    self.store_lane(idx, memarg, lane, Lang::V128Store32Lane)
+                }
+                Operator::V128Store64Lane { memarg, lane } => {
+                    self.store_lane(idx, memarg, lane, Lang::V128Store64Lane)
+                }
+
+                Operator::I8x16ExtractLaneS { lane } => {
+                    self.extract_lane(idx, lane, Lang::I8x16ExtractLaneS)
+                }
+                Operator::I8x16ExtractLaneU { lane } => {
+                    self.extract_lane(idx, lane, Lang::I8x16ExtractLaneU)
+                }
+                Operator::I8x16ReplaceLane { lane } => {
+                    self.replace_lane(idx, lane, Lang::I8x16ReplaceLane)
+                }
+                Operator::I16x8ExtractLaneS { lane } => {
+                    self.extract_lane(idx, lane, Lang::I16x8ExtractLaneS)
+                }
+                Operator::I16x8ExtractLaneU { lane } => {
+                    self.extract_lane(idx, lane, Lang::I16x8ExtractLaneU)
+                }
+                Operator::I16x8ReplaceLane { lane } => {
+                    self.replace_lane(idx, lane, Lang::I16x8ReplaceLane)
+                }
+                Operator::I32x4ExtractLane { lane } => {
+                    self.extract_lane(idx, lane, Lang::I32x4ExtractLane)
+                }
+                Operator::I32x4ReplaceLane { lane } => {
+                    self.replace_lane(idx, lane, Lang::I32x4ReplaceLane)
+                }
+                Operator::I64x2ExtractLane { lane } => {
+                    self.extract_lane(idx, lane, Lang::I64x2ExtractLane)
+                }
+                Operator::I64x2ReplaceLane { lane } => {
+                    self.replace_lane(idx, lane, Lang::I64x2ReplaceLane)
+                }
+                Operator::F32x4ExtractLane { lane } => {
+                    self.extract_lane(idx, lane, Lang::F32x4ExtractLane)
+                }
+                Operator::F32x4ReplaceLane { lane } => {
+                    self.replace_lane(idx, lane, Lang::F32x4ReplaceLane)
+                }
+                Operator::F64x2ExtractLane { lane } => {
+                    self.extract_lane(idx, lane, Lang::F64x2ExtractLane)
+                }
+                Operator::F64x2ReplaceLane { lane } => {
+                    self.replace_lane(idx, lane, Lang::F64x2ReplaceLane)
+                }
+
+                Operator::I8x16Swizzle => self.binop(idx, Lang::I8x16Swizzle),
+                Operator::I8x16Splat => self.unop(idx, Lang::I8x16Splat),
+                Operator::I16x8Splat => self.unop(idx, Lang::I16x8Splat),
+                Operator::I32x4Splat => self.unop(idx, Lang::I32x4Splat),
+                Operator::I64x2Splat => self.unop(idx, Lang::I64x2Splat),
+                Operator::F32x4Splat => self.unop(idx, Lang::F32x4Splat),
+                Operator::F64x2Splat => self.unop(idx, Lang::F64x2Splat),
+
+                Operator::I8x16Eq => self.binop(idx, Lang::I8x16Eq),
+                Operator::I8x16Ne => self.binop(idx, Lang::I8x16Ne),
+                Operator::I8x16LtS => self.binop(idx, Lang::I8x16LtS),
+                Operator::I8x16LtU => self.binop(idx, Lang::I8x16LtU),
+                Operator::I8x16GtS => self.binop(idx, Lang::I8x16GtS),
+                Operator::I8x16GtU => self.binop(idx, Lang::I8x16GtU),
+                Operator::I8x16LeS => self.binop(idx, Lang::I8x16LeS),
+                Operator::I8x16LeU => self.binop(idx, Lang::I8x16LeU),
+                Operator::I8x16GeS => self.binop(idx, Lang::I8x16GeS),
+                Operator::I8x16GeU => self.binop(idx, Lang::I8x16GeU),
+                Operator::I16x8Eq => self.binop(idx, Lang::I16x8Eq),
+                Operator::I16x8Ne => self.binop(idx, Lang::I16x8Ne),
+                Operator::I16x8LtS => self.binop(idx, Lang::I16x8LtS),
+                Operator::I16x8LtU => self.binop(idx, Lang::I16x8LtU),
+                Operator::I16x8GtS => self.binop(idx, Lang::I16x8GtS),
+                Operator::I16x8GtU => self.binop(idx, Lang::I16x8GtU),
+                Operator::I16x8LeS => self.binop(idx, Lang::I16x8LeS),
+                Operator::I16x8LeU => self.binop(idx, Lang::I16x8LeU),
+                Operator::I16x8GeS => self.binop(idx, Lang::I16x8GeS),
+                Operator::I16x8GeU => self.binop(idx, Lang::I16x8GeU),
+                Operator::I32x4Eq => self.binop(idx, Lang::I32x4Eq),
+                Operator::I32x4Ne => self.binop(idx, Lang::I32x4Ne),
+                Operator::I32x4LtS => self.binop(idx, Lang::I32x4LtS),
+                Operator::I32x4LtU => self.binop(idx, Lang::I32x4LtU),
+                Operator::I32x4GtS => self.binop(idx, Lang::I32x4GtS),
+                Operator::I32x4GtU => self.binop(idx, Lang::I32x4GtU),
+                Operator::I32x4LeS => self.binop(idx, Lang::I32x4LeS),
+                Operator::I32x4LeU => self.binop(idx, Lang::I32x4LeU),
+                Operator::I32x4GeS => self.binop(idx, Lang::I32x4GeS),
+                Operator::I32x4GeU => self.binop(idx, Lang::I32x4GeU),
+                Operator::I64x2Eq => self.binop(idx, Lang::I64x2Eq),
+                Operator::I64x2Ne => self.binop(idx, Lang::I64x2Ne),
+                Operator::I64x2LtS => self.binop(idx, Lang::I64x2LtS),
+                Operator::I64x2GtS => self.binop(idx, Lang::I64x2GtS),
+                Operator::I64x2LeS => self.binop(idx, Lang::I64x2LeS),
+                Operator::I64x2GeS => self.binop(idx, Lang::I64x2GeS),
+                Operator::F32x4Eq => self.binop(idx, Lang::F32x4Eq),
+                Operator::F32x4Ne => self.binop(idx, Lang::F32x4Ne),
+                Operator::F32x4Lt => self.binop(idx, Lang::F32x4Lt),
+                Operator::F32x4Gt => self.binop(idx, Lang::F32x4Gt),
+                Operator::F32x4Le => self.binop(idx, Lang::F32x4Le),
+                Operator::F32x4Ge => self.binop(idx, Lang::F32x4Ge),
+                Operator::F64x2Eq => self.binop(idx, Lang::F64x2Eq),
+                Operator::F64x2Ne => self.binop(idx, Lang::F64x2Ne),
+                Operator::F64x2Lt => self.binop(idx, Lang::F64x2Lt),
+                Operator::F64x2Gt => self.binop(idx, Lang::F64x2Gt),
+                Operator::F64x2Le => self.binop(idx, Lang::F64x2Le),
+                Operator::F64x2Ge => self.binop(idx, Lang::F64x2Ge),
+
+                Operator::I8x16Abs => self.unop(idx, Lang::I8x16Abs),
+                Operator::I8x16Neg => self.unop(idx, Lang::I8x16Neg),
+                Operator::I8x16Popcnt => self.unop(idx, Lang::I8x16Popcnt),
+                Operator::I8x16AllTrue => self.unop(idx, Lang::I8x16AllTrue),
+                Operator::I8x16Bitmask => self.unop(idx, Lang::I8x16Bitmask),
+                Operator::I8x16NarrowI16x8S => self.binop(idx, Lang::I8x16NarrowI16x8S),
+                Operator::I8x16NarrowI16x8U => self.binop(idx, Lang::I8x16NarrowI16x8U),
+                Operator::I8x16Shl => self.binop(idx, Lang::I8x16Shl),
+                Operator::I8x16ShrS => self.binop(idx, Lang::I8x16ShrS),
+                Operator::I8x16ShrU => self.binop(idx, Lang::I8x16ShrU),
+                Operator::I8x16Add => self.binop(idx, Lang::I8x16Add),
+                Operator::I8x16AddSatS => self.binop(idx, Lang::I8x16AddSatS),
+                Operator::I8x16AddSatU => self.binop(idx, Lang::I8x16AddSatU),
+                Operator::I8x16Sub => self.binop(idx, Lang::I8x16Sub),
+                Operator::I8x16SubSatS => self.binop(idx, Lang::I8x16SubSatS),
+                Operator::I8x16SubSatU => self.binop(idx, Lang::I8x16SubSatU),
+                Operator::I8x16MinS => self.binop(idx, Lang::I8x16MinS),
+                Operator::I8x16MinU => self.binop(idx, Lang::I8x16MinU),
+                Operator::I8x16MaxS => self.binop(idx, Lang::I8x16MaxS),
+                Operator::I8x16MaxU => self.binop(idx, Lang::I8x16MaxU),
+                Operator::I8x16RoundingAverageU => self.binop(idx, Lang::I8x16AvgrU),
+
+                Operator::I16x8ExtAddPairwiseI8x16S => {
+                    self.unop(idx, Lang::I16x8ExtAddPairwiseI8x16S)
+                }
+                Operator::I16x8ExtAddPairwiseI8x16U => {
+                    self.unop(idx, Lang::I16x8ExtAddPairwiseI8x16U)
+                }
+                Operator::I16x8Abs => self.unop(idx, Lang::I16x8Abs),
+                Operator::I16x8Neg => self.unop(idx, Lang::I16x8Neg),
+                Operator::I16x8Q15MulrSatS => self.binop(idx, Lang::I16x8Q15MulrSatS),
+                Operator::I16x8AllTrue => self.unop(idx, Lang::I16x8AllTrue),
+                Operator::I16x8Bitmask => self.unop(idx, Lang::I16x8Bitmask),
+                Operator::I16x8NarrowI32x4S => self.binop(idx, Lang::I16x8NarrowI32x4S),
+                Operator::I16x8NarrowI32x4U => self.binop(idx, Lang::I16x8NarrowI32x4U),
+                Operator::I16x8ExtendLowI8x16S => self.unop(idx, Lang::I16x8ExtendLowI8x16S),
+                Operator::I16x8ExtendHighI8x16S => self.unop(idx, Lang::I16x8ExtendHighI8x16S),
+                Operator::I16x8ExtendLowI8x16U => self.unop(idx, Lang::I16x8ExtendLowI8x16U),
+                Operator::I16x8ExtendHighI8x16U => self.unop(idx, Lang::I16x8ExtendHighI8x16U),
+                Operator::I16x8Shl => self.binop(idx, Lang::I16x8Shl),
+                Operator::I16x8ShrS => self.binop(idx, Lang::I16x8ShrS),
+                Operator::I16x8ShrU => self.binop(idx, Lang::I16x8ShrU),
+                Operator::I16x8Add => self.binop(idx, Lang::I16x8Add),
+                Operator::I16x8AddSatS => self.binop(idx, Lang::I16x8AddSatS),
+                Operator::I16x8AddSatU => self.binop(idx, Lang::I16x8AddSatU),
+                Operator::I16x8Sub => self.binop(idx, Lang::I16x8Sub),
+                Operator::I16x8SubSatS => self.binop(idx, Lang::I16x8SubSatS),
+                Operator::I16x8SubSatU => self.binop(idx, Lang::I16x8SubSatU),
+                Operator::I16x8Mul => self.binop(idx, Lang::I16x8Mul),
+                Operator::I16x8MinS => self.binop(idx, Lang::I16x8MinS),
+                Operator::I16x8MinU => self.binop(idx, Lang::I16x8MinU),
+                Operator::I16x8MaxS => self.binop(idx, Lang::I16x8MaxS),
+                Operator::I16x8MaxU => self.binop(idx, Lang::I16x8MaxU),
+                Operator::I16x8RoundingAverageU => self.binop(idx, Lang::I16x8AvgrU),
+                Operator::I16x8ExtMulLowI8x16S => self.binop(idx, Lang::I16x8ExtMulLowI8x16S),
+                Operator::I16x8ExtMulHighI8x16S => self.binop(idx, Lang::I16x8ExtMulHighI8x16S),
+                Operator::I16x8ExtMulLowI8x16U => self.binop(idx, Lang::I16x8ExtMulLowI8x16U),
+                Operator::I16x8ExtMulHighI8x16U => self.binop(idx, Lang::I16x8ExtMulHighI8x16U),
+
+                Operator::I32x4ExtAddPairwiseI16x8S => {
+                    self.unop(idx, Lang::I32x4ExtAddPairwiseI16x8S)
+                }
+                Operator::I32x4ExtAddPairwiseI16x8U => {
+                    self.unop(idx, Lang::I32x4ExtAddPairwiseI16x8U)
+                }
+                Operator::I32x4Abs => self.unop(idx, Lang::I32x4Abs),
+                Operator::I32x4Neg => self.unop(idx, Lang::I32x4Neg),
+                Operator::I32x4AllTrue => self.unop(idx, Lang::I32x4AllTrue),
+                Operator::I32x4Bitmask => self.unop(idx, Lang::I32x4Bitmask),
+                Operator::I32x4ExtendLowI16x8S => self.unop(idx, Lang::I32x4ExtendLowI16x8S),
+                Operator::I32x4ExtendHighI16x8S => self.unop(idx, Lang::I32x4ExtendHighI16x8S),
+                Operator::I32x4ExtendLowI16x8U => self.unop(idx, Lang::I32x4ExtendLowI16x8U),
+                Operator::I32x4ExtendHighI16x8U => self.unop(idx, Lang::I32x4ExtendHighI16x8U),
+                Operator::I32x4Shl => self.binop(idx, Lang::I32x4Shl),
+                Operator::I32x4ShrS => self.binop(idx, Lang::I32x4ShrS),
+                Operator::I32x4ShrU => self.binop(idx, Lang::I32x4ShrU),
+                Operator::I32x4Add => self.binop(idx, Lang::I32x4Add),
+                Operator::I32x4Sub => self.binop(idx, Lang::I32x4Sub),
+                Operator::I32x4Mul => self.binop(idx, Lang::I32x4Mul),
+                Operator::I32x4MinS => self.binop(idx, Lang::I32x4MinS),
+                Operator::I32x4MinU => self.binop(idx, Lang::I32x4MinU),
+                Operator::I32x4MaxS => self.binop(idx, Lang::I32x4MaxS),
+                Operator::I32x4MaxU => self.binop(idx, Lang::I32x4MaxU),
+                Operator::I32x4DotI16x8S => self.binop(idx, Lang::I32x4DotI16x8S),
+                Operator::I32x4ExtMulLowI16x8S => self.binop(idx, Lang::I32x4ExtMulLowI16x8S),
+                Operator::I32x4ExtMulHighI16x8S => self.binop(idx, Lang::I32x4ExtMulHighI16x8S),
+                Operator::I32x4ExtMulLowI16x8U => self.binop(idx, Lang::I32x4ExtMulLowI16x8U),
+                Operator::I32x4ExtMulHighI16x8U => self.binop(idx, Lang::I32x4ExtMulHighI16x8U),
+
+                Operator::I64x2Abs => self.unop(idx, Lang::I64x2Abs),
+                Operator::I64x2Neg => self.unop(idx, Lang::I64x2Neg),
+                Operator::I64x2AllTrue => self.unop(idx, Lang::I64x2AllTrue),
+                Operator::I64x2Bitmask => self.unop(idx, Lang::I64x2Bitmask),
+                Operator::I64x2ExtendLowI32x4S => self.unop(idx, Lang::I64x2ExtendLowI32x4S),
+                Operator::I64x2ExtendHighI32x4S => self.unop(idx, Lang::I64x2ExtendHighI32x4S),
+                Operator::I64x2ExtendLowI32x4U => self.unop(idx, Lang::I64x2ExtendLowI32x4U),
+                Operator::I64x2ExtendHighI32x4U => self.unop(idx, Lang::I64x2ExtendHighI32x4U),
+                Operator::I64x2Shl => self.binop(idx, Lang::I64x2Shl),
+                Operator::I64x2ShrS => self.binop(idx, Lang::I64x2ShrS),
+                Operator::I64x2ShrU => self.binop(idx, Lang::I64x2ShrU),
+                Operator::I64x2Add => self.binop(idx, Lang::I64x2Add),
+                Operator::I64x2Sub => self.binop(idx, Lang::I64x2Sub),
+                Operator::I64x2Mul => self.binop(idx, Lang::I64x2Mul),
+                Operator::I64x2ExtMulLowI32x4S => self.binop(idx, Lang::I64x2ExtMulLowI32x4S),
+                Operator::I64x2ExtMulHighI32x4S => self.binop(idx, Lang::I64x2ExtMulHighI32x4S),
+                Operator::I64x2ExtMulLowI32x4U => self.binop(idx, Lang::I64x2ExtMulLowI32x4U),
+                Operator::I64x2ExtMulHighI32x4U => self.binop(idx, Lang::I64x2ExtMulHighI32x4U),
+
+                Operator::F32x4Ceil => self.unop(idx, Lang::F32x4Ceil),
+                Operator::F32x4Floor => self.unop(idx, Lang::F32x4Floor),
+                Operator::F32x4Trunc => self.unop(idx, Lang::F32x4Trunc),
+                Operator::F32x4Nearest => self.unop(idx, Lang::F32x4Nearest),
+                Operator::F32x4Abs => self.unop(idx, Lang::F32x4Abs),
+                Operator::F32x4Neg => self.unop(idx, Lang::F32x4Neg),
+                Operator::F32x4Sqrt => self.unop(idx, Lang::F32x4Sqrt),
+                Operator::F32x4Add => self.binop(idx, Lang::F32x4Add),
+                Operator::F32x4Sub => self.binop(idx, Lang::F32x4Sub),
+                Operator::F32x4Mul => self.binop(idx, Lang::F32x4Mul),
+                Operator::F32x4Div => self.binop(idx, Lang::F32x4Div),
+                Operator::F32x4Min => self.binop(idx, Lang::F32x4Min),
+                Operator::F32x4Max => self.binop(idx, Lang::F32x4Max),
+                Operator::F32x4PMin => self.binop(idx, Lang::F32x4PMin),
+                Operator::F32x4PMax => self.binop(idx, Lang::F32x4PMax),
+                Operator::F64x2Ceil => self.unop(idx, Lang::F64x2Ceil),
+                Operator::F64x2Floor => self.unop(idx, Lang::F64x2Floor),
+                Operator::F64x2Trunc => self.unop(idx, Lang::F64x2Trunc),
+                Operator::F64x2Nearest => self.unop(idx, Lang::F64x2Nearest),
+                Operator::F64x2Abs => self.unop(idx, Lang::F64x2Abs),
+                Operator::F64x2Neg => self.unop(idx, Lang::F64x2Neg),
+                Operator::F64x2Sqrt => self.unop(idx, Lang::F64x2Sqrt),
+                Operator::F64x2Add => self.binop(idx, Lang::F64x2Add),
+                Operator::F64x2Sub => self.binop(idx, Lang::F64x2Sub),
+                Operator::F64x2Mul => self.binop(idx, Lang::F64x2Mul),
+                Operator::F64x2Div => self.binop(idx, Lang::F64x2Div),
+                Operator::F64x2Min => self.binop(idx, Lang::F64x2Min),
+                Operator::F64x2Max => self.binop(idx, Lang::F64x2Max),
+                Operator::F64x2PMin => self.binop(idx, Lang::F64x2PMin),
+                Operator::F64x2PMax => self.binop(idx, Lang::F64x2PMax),
+
+                Operator::I32x4TruncSatF32x4S => self.unop(idx, Lang::I32x4TruncSatF32x4S),
+                Operator::I32x4TruncSatF32x4U => self.unop(idx, Lang::I32x4TruncSatF32x4U),
+                Operator::F32x4ConvertI32x4S => self.unop(idx, Lang::F32x4ConvertI32x4S),
+                Operator::F32x4ConvertI32x4U => self.unop(idx, Lang::F32x4ConvertI32x4U),
+                Operator::I32x4TruncSatF64x2SZero => self.unop(idx, Lang::I32x4TruncSatF64x2SZero),
+                Operator::I32x4TruncSatF64x2UZero => self.unop(idx, Lang::I32x4TruncSatF64x2UZero),
+                Operator::F64x2ConvertLowI32x4S => self.unop(idx, Lang::F64x2ConvertLowI32x4S),
+                Operator::F64x2ConvertLowI32x4U => self.unop(idx, Lang::F64x2ConvertLowI32x4U),
+                Operator::F32x4DemoteF64x2Zero => self.unop(idx, Lang::F32x4DemoteF64x2Zero),
+                Operator::F64x2PromoteLowF32x4 => self.unop(idx, Lang::F64x2PromoteLowF32x4),
+
+                op => {
+                    // If the operator is not implemented, warn and bail out. We
+                    // can't use `undef` for these because if we try to rewrite
+                    // them with some rule like `x => i32.rand` then we fail to
+                    // actually encode the rewritten code to valid Wasm right
+                    // now. We would need to be able to walk the new expression
+                    // and determine which `undef` values became dead code and
+                    // we need to insert `drop`s for before inserting the new
+                    // code, and we don't currently have that infrastructure.
+                    log::warn!("Wasm operator not implemented: {:?}", op);
                     return None;
                 }
             }
@@ -765,6 +1111,30 @@ impl<'a> DFGBuilder {
         self.unop_cb(idx, |id| op(memarg.into(), id))
     }
 
+    fn load_lane(
+        &mut self,
+        idx: usize,
+        memarg: &wasmparser::MemoryImmediate,
+        lane: &u8,
+        op: fn(MemArgLane, [Id; 2]) -> Lang,
+    ) {
+        let leftidx = self.pop_operand(idx, false);
+        let rightidx = self.pop_operand(idx, false);
+        // The operands should not be the same
+        assert_ne!(leftidx, rightidx);
+
+        self.push_node(
+            op(
+                MemArgLane {
+                    lane: *lane,
+                    memarg: memarg.into(),
+                },
+                [Id::from(rightidx), Id::from(leftidx)],
+            ),
+            idx,
+        );
+    }
+
     fn unop_cb(&mut self, idx: usize, op: impl Fn(Id) -> Lang) {
         let arg = self.pop_operand(idx, false);
         self.push_node(op(Id::from(arg)), idx);
@@ -780,6 +1150,19 @@ impl<'a> DFGBuilder {
         self.push_node(op([Id::from(rightidx), Id::from(leftidx)]), idx);
     }
 
+    fn ternop(&mut self, idx: usize, op: fn([Id; 3]) -> Lang) {
+        let c = self.pop_operand(idx, false);
+        let b = self.pop_operand(idx, false);
+        let a = self.pop_operand(idx, false);
+
+        // The operands should not be the same.
+        assert_ne!(a, b);
+        assert_ne!(a, c);
+        assert_ne!(b, c);
+
+        self.push_node(op([Id::from(a), Id::from(b), Id::from(c)]), idx);
+    }
+
     fn store(
         &mut self,
         idx: usize,
@@ -797,6 +1180,42 @@ impl<'a> DFGBuilder {
             idx,
         );
     }
+
+    fn store_lane(
+        &mut self,
+        idx: usize,
+        memarg: &wasmparser::MemoryImmediate,
+        lane: &u8,
+        op: fn(MemArgLane, [Id; 2]) -> Lang,
+    ) {
+        let leftidx = self.pop_operand(idx, false);
+        let rightidx = self.pop_operand(idx, false);
+
+        // The operands should not be the same
+        assert_ne!(leftidx, rightidx);
+
+        self.empty_node(
+            op(
+                MemArgLane {
+                    lane: *lane,
+                    memarg: memarg.into(),
+                },
+                [Id::from(rightidx), Id::from(leftidx)],
+            ),
+            idx,
+        );
+    }
+
+    fn extract_lane(&mut self, idx: usize, lane: &u8, op: fn(u8, Id) -> Lang) {
+        let arg = self.pop_operand(idx, false);
+        self.push_node(op(*lane, Id::from(arg)), idx);
+    }
+
+    fn replace_lane(&mut self, idx: usize, lane: &u8, op: fn(u8, [Id; 2]) -> Lang) {
+        let leftidx = self.pop_operand(idx, false);
+        let rightidx = self.pop_operand(idx, false);
+        self.push_node(op(*lane, [Id::from(rightidx), Id::from(leftidx)]), idx);
+    }
 }
 
 impl std::fmt::Debug for MiniDFG {
@@ -825,7 +1244,7 @@ impl From<&wasmparser::MemoryImmediate> for MemArg {
 mod tests {
     use super::DFGBuilder;
     use crate::mutators::OperatorAndByteOffset;
-    use crate::ModuleInfo;
+    use crate::{ModuleInfo, WasmMutate};
     use wasmparser::Parser;
 
     #[test]
@@ -860,6 +1279,7 @@ mod tests {
         )
         .unwrap();
 
+        let config = WasmMutate::default();
         let mut parser = Parser::new(0);
         let mut consumed = 0;
         loop {
@@ -881,11 +1301,11 @@ mod tests {
                         .collect::<wasmparser::Result<Vec<OperatorAndByteOffset>>>()
                         .unwrap();
 
-                    let roots = DFGBuilder::new().get_bb_from_operator(5, &operators);
+                    let roots = DFGBuilder::new(&config).get_bb_from_operator(5, &operators);
 
                     assert!(roots.is_some())
                 }
-                wasmparser::Payload::End => {
+                wasmparser::Payload::End(_) => {
                     break;
                 }
                 _ => {
@@ -915,6 +1335,7 @@ mod tests {
         )
         .unwrap();
 
+        let config = WasmMutate::default();
         let mut parser = Parser::new(0);
         let mut consumed = 0;
         loop {
@@ -936,13 +1357,14 @@ mod tests {
                         .collect::<wasmparser::Result<Vec<OperatorAndByteOffset>>>()
                         .unwrap();
 
-                    let bb = DFGBuilder::new()
+                    let bb = DFGBuilder::new(&config)
                         .get_bb_from_operator(0, &operators)
                         .unwrap();
-                    let roots = DFGBuilder::new().get_dfg(&ModuleInfo::default(), &operators, &bb);
+                    let roots =
+                        DFGBuilder::new(&config).get_dfg(&ModuleInfo::default(), &operators, &bb);
                     assert!(roots.is_some())
                 }
-                wasmparser::Payload::End => {
+                wasmparser::Payload::End(_) => {
                     break;
                 }
                 _ => {
@@ -995,6 +1417,7 @@ mod tests {
         )
         .unwrap();
 
+        let config = WasmMutate::default();
         let mut parser = Parser::new(0);
         let mut consumed = 0;
         loop {
@@ -1016,13 +1439,14 @@ mod tests {
                         .collect::<wasmparser::Result<Vec<OperatorAndByteOffset>>>()
                         .unwrap();
 
-                    let bb = DFGBuilder::new()
+                    let bb = DFGBuilder::new(&config)
                         .get_bb_from_operator(7, &operators)
                         .unwrap();
-                    let roots = DFGBuilder::new().get_dfg(&ModuleInfo::default(), &operators, &bb);
+                    let roots =
+                        DFGBuilder::new(&config).get_dfg(&ModuleInfo::default(), &operators, &bb);
                     assert!(roots.is_some());
                 }
-                wasmparser::Payload::End => {
+                wasmparser::Payload::End(_) => {
                     break;
                 }
                 _ => {
@@ -1050,6 +1474,7 @@ mod tests {
         )
         .unwrap();
 
+        let config = WasmMutate::default();
         let info = ModuleInfo::new(original).unwrap();
 
         let mut parser = Parser::new(0);
@@ -1071,13 +1496,13 @@ mod tests {
                         .collect::<wasmparser::Result<Vec<OperatorAndByteOffset>>>()
                         .unwrap();
 
-                    let bb = DFGBuilder::new()
+                    let bb = DFGBuilder::new(&config)
                         .get_bb_from_operator(3, &operators)
                         .unwrap();
-                    let roots = DFGBuilder::new().get_dfg(&info, &operators, &bb);
+                    let roots = DFGBuilder::new(&config).get_dfg(&info, &operators, &bb);
                     assert!(roots.is_some());
                 }
-                wasmparser::Payload::End => {
+                wasmparser::Payload::End(_) => {
                     break;
                 }
                 _ => {
