@@ -415,6 +415,16 @@ impl PeepholeMutator {
                     "?x" => "i64.const.1"
                         if self.is_type("?x", PrimitiveTypeInfo::I64)
                 ),
+                rewrite!(
+                    "replace-with-f32-1.0";
+                    "?x" => "f32.const.1065353216"
+                        if self.is_type("?x", PrimitiveTypeInfo::F32)
+                ),
+                rewrite!(
+                    "replace-with-f64-1.0";
+                    "?x" => "f64.const.4607182418800017408"
+                        if self.is_type("?x", PrimitiveTypeInfo::F64)
+                ),
                 rewrite!(
                     "replace-with-i32-0";
                     "?x" => "i32.const.0"
@@ -425,6 +435,21 @@ impl PeepholeMutator {
                     "?x" => "i64.const.0"
                         if self.is_type("?x", PrimitiveTypeInfo::I64)
                 ),
+                rewrite!(
+                    "replace-with-f32-0";
+                    "?x" => "f32.const.0"
+                        if self.is_type("?x", PrimitiveTypeInfo::F32)
+                ),
+                rewrite!(
+                    "replace-with-f64-0";
+                    "?x" => "f64.const.0"
+                        if self.is_type("?x", PrimitiveTypeInfo::F64)
+                ),
+                rewrite!(
+                    "replace-with-v128-0";
+                    "?x" => "v128.const.0"
+                        if self.is_type("?x", PrimitiveTypeInfo::V128)
+                ),
                 rewrite!(
                     "replace-with-i32.rand";
                     "?x" => "i32.rand"
@@ -435,6 +460,60 @@ impl PeepholeMutator {
                     "?x" => "i64.rand"
                         if self.is_type("?x", PrimitiveTypeInfo::I64)
                 ),
+                rewrite!(
+                    "replace-with-f32.rand";
+                    "?x" => "f32.rand"
+                        if self.is_type("?x", PrimitiveTypeInfo::F32)
+                ),
+                rewrite!(
+                    "replace-with-f64.rand";
+                    "?x" => "f64.rand"
+                        if self.is_type("?x", PrimitiveTypeInfo::F64)
+                ),
+                rewrite!(
+                    "replace-with-ref-null-func";
+                    "?x" => "ref.null.func"
+                        if self.is_type("?x", PrimitiveTypeInfo::FuncRef)
+                ),
+                rewrite!(
+                    "replace-with-ref-null-extern";
+                    "?x" => "ref.null.extern"
+                        if self.is_type("?x", PrimitiveTypeInfo::ExternRef)
+                ),
+                // Try to outright delete some instructions and containers to
+                // help make input even smaller in wasm-shrink cases.
+                rewrite!(
+                    "remove-drop";
+                    "(drop ?x)" => "(container)"
+                ),
+                rewrite!(
+                    "remove-nop";
+                    "nop" => "(container)"
+                ),
+                rewrite!(
+                    "remove-global.set.0";
+                    "(global.set.0 ?x)" => "(container)"
+                ),
+                rewrite!(
+                    "remove-global.set.1";
+                    "(global.set.1 ?x)" => "(container)"
+                ),
+                rewrite!(
+                    "remove-elem.drop.0";
+                    "(elem.drop.0)" => "(container)"
+                ),
+                rewrite!(
+                    "remove-elem.drop.1";
+                    "(elem.drop.1)" => "(container)"
+                ),
+                rewrite!(
+                    "remove-data.drop.0";
+                    "(data.drop.0)" => "(container)"
+                ),
+                rewrite!(
+                    "remove-data.drop.1";
+                    "(data.drop.1)" => "(container)"
+                ),
             ]);
 
             if !config.reduce {
