@@ -13,7 +13,12 @@ use crate::{ModuleInfo, Result, WasmMutate};
 use rand::Rng;
 use std::collections::HashSet;
 use wasm_encoder::*;
-use wasmparser::*;
+use wasmparser::{
+    BinaryReader, CodeSectionReader, DataSectionReader, ElementSectionReader, ExportSectionReader,
+    ExternalKind, FunctionSectionReader, GlobalSectionReader, ImportSectionReader,
+    MemorySectionReader, Operator, SectionReader, TableSectionReader, TagSectionReader,
+    TypeSectionReader,
+};
 
 /// Mutator that removes a random item in a wasm module (function, global,
 /// table, etc).
@@ -41,9 +46,17 @@ impl Mutator for RemoveItemMutator {
             referenced_functions: HashSet::new(),
             function_reference_action: Funcref::Save,
         }
-        .remove(config.info())?;
-        log::debug!("removed {:?} index {}", self.0, idx);
-        Ok(Box::new(std::iter::once(Ok(result))))
+        .remove(config.info());
+        match result {
+            Ok(result) => {
+                log::debug!("removed {:?} index {}", self.0, idx);
+                Ok(Box::new(std::iter::once(Ok(result))))
+            }
+            Err(e) => {
+                log::trace!("failed to remove {:?} index {}: {:?}", self.0, idx, e);
+                Err(e)
+            }
+        }
     }
 }
 
@@ -107,35 +120,19 @@ enum Funcref {
 
 impl RemoveItem {
     fn remove(&mut self, info: &ModuleInfo) -> Result<Module> {
-        const CUSTOM: u8 = SectionId::Custom as u8;
-        const TYPE: u8 = SectionId::Type as u8;
-        const IMPORT: u8 = SectionId::Import as u8;
-        const FUNCTION: u8 = SectionId::Function as u8;
-        const TABLE: u8 = SectionId::Table as u8;
-        const MEMORY: u8 = SectionId::Memory as u8;
-        const GLOBAL: u8 = SectionId::Global as u8;
-        const EXPORT: u8 = SectionId::Export as u8;
-        const START: u8 = SectionId::Start as u8;
-        const ELEMENT: u8 = SectionId::Element as u8;
-        const CODE: u8 = SectionId::Code as u8;
-        const DATA: u8 = SectionId::Data as u8;
-        const DATACOUNT: u8 = SectionId::DataCount as u8;
-        const TAG: u8 = SectionId::Tag as u8;
-        const MODULE: u8 = SectionId::Module as u8;
-        const INSTANCE: u8 = SectionId::Instance as u8;
-        const ALIAS: u8 = SectionId::Alias as u8;
-
         // This is the main workhorse loop of the module translation. This will
         // iterate over the original wasm sections, raw, and create the new
         // module section-by-section. Sections are rewritten on-the-fly.
         let mut module = Module::new();
         for section in info.raw_sections.iter() {
-            match section.id {
-                CUSTOM => {
-                    module.section(section);
-                }
+            crate::module::match_section_id! {
+                match section.id;
 
-                TYPE => {
+                Custom => {
+                    module.section(section);
+                },
+
+                Type => {
                     self.filter_out(
                         &mut module,
                         0,
@@ -143,14 +140,14 @@ impl RemoveItem {
                         Item::Type,
                         |me, ty, section| me.translate_type_def(ty, section),
                     )?;
-                }
+                },
 
-                // The import section is a little special because it defines
-                // items in multiple index spaces. This means that the
-                // `filter_out` helper can't be used and we have to process
-                // everything manually here.
-                IMPORT => {
-                    let mut result = ImportSection::default();
+                Import => {
+                    // The import section is a little special because it defines
+                    // items in multiple index spaces. This means that the
+                    // `filter_out` helper can't be used and we have to process
+                    // everything manually here.
+                    let mut result = ImportSection::new();
                     let mut function = 0;
                     let mut global = 0;
                     let mut table = 0;
@@ -159,55 +156,47 @@ impl RemoveItem {
                     for item in ImportSectionReader::new(section.data, 0)? {
                         let item = item?;
                         match &item.ty {
-                            ImportSectionEntryType::Function(ty) => {
+                            wasmparser::TypeRef::Func(ty) => {
                                 if self.item != Item::Function || self.idx != function {
                                     let ty = self.remap(Item::Type, *ty)?;
-                                    result.import(
-                                        item.module,
-                                        item.field,
-                                        EntityType::Function(ty),
-                                    );
+                                    result.import(item.module, item.name, EntityType::Function(ty));
                                 }
                                 function += 1;
                             }
-                            ImportSectionEntryType::Table(ty) => {
+                            wasmparser::TypeRef::Table(ty) => {
                                 if self.item != Item::Table || self.idx != table {
                                     let ty = self.translate_table_type(ty)?;
-                                    result.import(item.module, item.field, ty);
+                                    result.import(item.module, item.name, ty);
                                 }
                                 table += 1;
                             }
-                            ImportSectionEntryType::Memory(ty) => {
+                            wasmparser::TypeRef::Memory(ty) => {
                                 if self.item != Item::Memory || self.idx != memory {
                                     let ty = self.translate_memory_type(ty)?;
-                                    result.import(item.module, item.field, ty);
+                                    result.import(item.module, item.name, ty);
                                 }
                                 memory += 1;
                             }
-                            ImportSectionEntryType::Global(ty) => {
+                            wasmparser::TypeRef::Global(ty) => {
                                 if self.item != Item::Global || self.idx != global {
                                     let ty = self.translate_global_type(ty)?;
-                                    result.import(item.module, item.field, ty);
+                                    result.import(item.module, item.name, ty);
                                 }
                                 global += 1;
                             }
-                            ImportSectionEntryType::Tag(ty) => {
+                            wasmparser::TypeRef::Tag(ty) => {
                                 if self.item != Item::Tag || self.idx != tag {
                                     let ty = self.translate_tag_type(ty)?;
-                                    result.import(item.module, item.field, ty);
+                                    result.import(item.module, item.name, ty);
                                 }
                                 tag += 1;
                             }
-                            ImportSectionEntryType::Instance(_)
-                            | ImportSectionEntryType::Module(_) => {
-                                return Err(Error::no_mutations_applicable())
-                            }
                         }
                     }
                     module.section(&result);
-                }
+                },
 
-                FUNCTION => {
+                Function => {
                     self.filter_out(
                         &mut module,
                         info.num_imported_functions(),
@@ -219,9 +208,9 @@ impl RemoveItem {
                             Ok(())
                         },
                     )?;
-                }
+                },
 
-                TABLE => {
+                Table => {
                     self.filter_out(
                         &mut module,
                         info.num_imported_tables(),
@@ -233,9 +222,9 @@ impl RemoveItem {
                             Ok(())
                         },
                     )?;
-                }
+                },
 
-                MEMORY => {
+                Memory => {
                     self.filter_out(
                         &mut module,
                         info.num_imported_memories(),
@@ -247,9 +236,9 @@ impl RemoveItem {
                             Ok(())
                         },
                     )?;
-                }
+                },
 
-                GLOBAL => {
+                Global => {
                     self.filter_out(
                         &mut module,
                         info.num_imported_globals(),
@@ -257,16 +246,14 @@ impl RemoveItem {
                         Item::Global,
                         |me, ty, section| me.translate_global(ty, section),
                     )?;
-                }
+                },
 
-                EXPORT => {
-                    use wasm_encoder::Export;
-
-                    let mut result = ExportSection::default();
+                Export => {
+                    let mut result = ExportSection::new();
                     for item in ExportSectionReader::new(section.data, 0)? {
                         let item = item?;
                         let e = match &item.kind {
-                            ExternalKind::Function => {
+                            ExternalKind::Func => {
                                 Export::Function(self.remap(Item::Function, item.index)?)
                             }
                             ExternalKind::Table => {
@@ -279,24 +266,21 @@ impl RemoveItem {
                             ExternalKind::Global => {
                                 Export::Global(self.remap(Item::Global, item.index)?)
                             }
-                            ExternalKind::Type | ExternalKind::Instance | ExternalKind::Module => {
-                                return Err(Error::no_mutations_applicable())
-                            }
                         };
-                        result.export(item.field, e);
+                        result.export(item.name, e);
                     }
                     module.section(&result);
-                }
+                },
 
-                START => {
+                Start => {
                     let function_index = BinaryReader::new(section.data).read_var_u32()?;
                     self.function_reference_action = Funcref::Skip;
                     let function_index = self.remap(Item::Function, function_index)?;
                     self.function_reference_action = Funcref::Save;
                     module.section(&StartSection { function_index });
-                }
+                },
 
-                ELEMENT => {
+                Element => {
                     self.filter_out(
                         &mut module,
                         0,
@@ -304,9 +288,9 @@ impl RemoveItem {
                         Item::Element,
                         |me, ty, section| me.translate_element(ty, section),
                     )?;
-                }
+                },
 
-                CODE => {
+                Code => {
                     // In the code section we require that all functions
                     // referenced in `ref.func` are referenced elsewhere in the
                     // module, so indicate so in our internal state here.
@@ -318,9 +302,9 @@ impl RemoveItem {
                         Item::Function,
                         |me, body, section| me.translate_code(body, section),
                     )?;
-                }
+                },
 
-                DATA => {
+                Data => {
                     self.filter_out(
                         &mut module,
                         0,
@@ -328,9 +312,9 @@ impl RemoveItem {
                         Item::Data,
                         |me, ty, section| me.translate_data(ty, section),
                     )?;
-                }
+                },
 
-                DATACOUNT => {
+                DataCount => {
                     let count = BinaryReader::new(section.data).read_var_u32()?;
                     // Note that the data count section is decremented here if
                     // we're removing a data item, otherwise it's preserved
@@ -341,9 +325,9 @@ impl RemoveItem {
                         count
                     };
                     module.section(&DataCountSection { count });
-                }
+                },
 
-                TAG => {
+                Tag => {
                     self.filter_out(
                         &mut module,
                         info.num_imported_tags(),
@@ -355,13 +339,10 @@ impl RemoveItem {
                             Ok(())
                         },
                     )?;
-                }
+                },
 
-                // Module linking is not supported at this time.
-                MODULE | INSTANCE | ALIAS => return Err(Error::no_mutations_applicable()),
-
-                id => panic!("unknown id: {}", id),
-            }
+                _ => panic!("unknown id: {}", section.id),
+            };
         }
         Ok(module)
     }
@@ -427,6 +408,15 @@ impl Translator for RemoveItem {
     /// * Finally our index is larger than the one being removed which means we
     ///   now decrement it by one to account for the removed item.
     fn remap(&mut self, item: Item, idx: u32) -> Result<u32> {
+        // If we're before the code section then all function references, no
+        // matter where they are, are considered "referencing functions" so we
+        // save the indices of that which is referenced.
+        if item == Item::Function {
+            if let Funcref::Save = self.function_reference_action {
+                self.referenced_functions.insert(idx);
+            }
+        }
+
         if item != self.item {
             // Different kind of item, no change
             Ok(idx)
@@ -457,19 +447,13 @@ impl Translator for RemoveItem {
         // otherwise referenced (probably because we removed the one
         // item that referenced it).
         if let Operator::RefFunc { function_index } = op {
-            let idx = *function_index;
-            match self.function_reference_action {
-                Funcref::Save => {
-                    self.referenced_functions.insert(idx);
-                }
-                Funcref::Skip => {}
-                Funcref::RequireReferenced => {
-                    if !self.referenced_functions.contains(&idx) {
-                        return Err(Error::no_mutations_applicable());
-                    }
+            if let Funcref::RequireReferenced = self.function_reference_action {
+                if !self.referenced_functions.contains(function_index) {
+                    return Err(Error::no_mutations_applicable());
                 }
             }
         }
+
         translate::op(self, op)
     }
 }
@@ -710,4 +694,46 @@ mod tests {
             )"#,
         );
     }
+
+    #[test]
+    fn remove_function_with_memarg64() {
+        crate::mutators::match_mutation(
+            r#"(module
+                    (memory i64 0)
+                    (func)
+                    (func (export "")
+                        i64.const 0
+                        i64.load offset=1125899906842624
+                        drop)
+            )"#,
+            RemoveItemMutator(Item::Function),
+            r#"(module
+                    (memory i64 0)
+                    (func (export "")
+                        i64.const 0
+                        i64.load offset=1125899906842624
+                        drop)
+            )"#,
+        );
+    }
+
+    #[test]
+    fn remove_empty_element() {
+        crate::mutators::match_mutation(
+            r#"(module
+                    (func (result funcref)
+                        ref.func 0)
+                    ;; cannot be removed otherwise the `ref.func 0` would be
+                    ;; inapplicable
+                    (elem declare func 0)
+                    (elem declare func)
+            )"#,
+            RemoveItemMutator(Item::Element),
+            r#"(module
+                    (func (result funcref)
+                        ref.func 0)
+                    (elem declare func 0)
+            )"#,
+        );
+    }
 }
