@@ -1,6 +1,9 @@
 use crate::{Error, Result};
 use wasm_encoder::*;
-use wasmparser::*;
+use wasmparser::{
+    DataKind, ElementItem, ElementKind, FunctionBody, Global, InitExpr, MemoryImmediate, Operator,
+    Type, TypeDef,
+};
 
 #[derive(Debug, Hash, Eq, PartialEq, Copy, Clone)]
 pub enum Item {
@@ -14,6 +17,14 @@ pub enum Item {
     Element,
 }
 
+#[derive(Debug, Hash, Eq, PartialEq, Copy, Clone)]
+pub enum InitExprKind {
+    Global,
+    ElementOffset,
+    ElementFunction,
+    DataOffset,
+}
+
 pub trait Translator {
     fn as_obj(&mut self) -> &mut dyn Translator;
 
@@ -54,7 +65,12 @@ pub trait Translator {
         global(self.as_obj(), g, s)
     }
 
-    fn translate_init_expr(&mut self, e: &InitExpr<'_>) -> Result<Instruction<'static>> {
+    fn translate_init_expr(
+        &mut self,
+        e: &InitExpr<'_>,
+        _ty: &Type,
+        _ctx: InitExprKind,
+    ) -> Result<Instruction<'static>> {
         init_expr(self.as_obj(), e)
     }
 
@@ -78,7 +94,7 @@ pub trait Translator {
         op(self.as_obj(), e)
     }
 
-    fn translate_block_type(&mut self, ty: &TypeOrFuncType) -> Result<BlockType> {
+    fn translate_block_type(&mut self, ty: &wasmparser::BlockType) -> Result<BlockType> {
         block_type(self.as_obj(), ty)
     }
 
@@ -114,9 +130,6 @@ pub fn type_def(t: &mut dyn Translator, ty: TypeDef, s: &mut TypeSection) -> Res
             );
             Ok(())
         }
-
-        // Module linking is not supported at this time.
-        TypeDef::Instance(_) | TypeDef::Module(_) => Err(Error::no_mutations_applicable()),
     }
 }
 
@@ -155,7 +168,7 @@ pub fn global_type(
 pub fn tag_type(t: &mut dyn Translator, ty: &wasmparser::TagType) -> Result<wasm_encoder::TagType> {
     Ok(wasm_encoder::TagType {
         kind: TagKind::Exception,
-        func_type_idx: t.remap(Item::Type, ty.type_index)?,
+        func_type_idx: t.remap(Item::Type, ty.func_type_idx)?,
     })
 }
 
@@ -168,19 +181,16 @@ pub fn ty(_t: &mut dyn Translator, ty: &Type) -> Result<ValType> {
         Type::V128 => Ok(ValType::V128),
         Type::FuncRef => Ok(ValType::FuncRef),
         Type::ExternRef => Ok(ValType::ExternRef),
-
-        // not supported in wasm-encoder
-        Type::ExnRef => Err(Error::no_mutations_applicable()),
-
-        // Shouldn't ever show up as these are used in different contexts
-        // within wasmparser.
-        Type::Func | Type::EmptyBlockType => Err(Error::no_mutations_applicable()),
     }
 }
 
 pub fn global(t: &mut dyn Translator, global: Global, s: &mut GlobalSection) -> Result<()> {
     let ty = t.translate_global_type(&global.ty)?;
-    let insn = t.translate_init_expr(&global.init_expr)?;
+    let insn = t.translate_init_expr(
+        &global.init_expr,
+        &global.ty.content_type,
+        InitExprKind::Global,
+    )?;
     s.global(ty, &insn);
     Ok(())
 }
@@ -207,7 +217,7 @@ pub fn element(
             table_index,
             init_expr,
         } => {
-            offset = t.translate_init_expr(init_expr)?;
+            offset = t.translate_init_expr(init_expr, &Type::I32, InitExprKind::ElementOffset)?;
             ElementMode::Active {
                 table: Some(t.remap(Item::Table, *table_index)?),
                 offset: &offset,
@@ -225,15 +235,17 @@ pub fn element(
             ElementItem::Func(idx) => {
                 functions.push(t.remap(Item::Function, idx)?);
             }
-            ElementItem::Expr(expr) => match t.translate_init_expr(&expr)? {
-                Instruction::RefFunc(n) => {
-                    exprs.push(wasm_encoder::Element::Func(n));
+            ElementItem::Expr(expr) => {
+                match t.translate_init_expr(&expr, &element.ty, InitExprKind::ElementFunction)? {
+                    Instruction::RefFunc(n) => {
+                        exprs.push(wasm_encoder::Element::Func(n));
+                    }
+                    Instruction::RefNull(_) => {
+                        exprs.push(wasm_encoder::Element::Null);
+                    }
+                    _ => return Err(Error::no_mutations_applicable()),
                 }
-                Instruction::RefNull(_) => {
-                    exprs.push(wasm_encoder::Element::Null);
-                }
-                _ => return Err(Error::no_mutations_applicable()),
-            },
+            }
         }
     }
     s.segment(ElementSegment {
@@ -281,7 +293,11 @@ pub fn op(t: &mut dyn Translator, op: &Operator<'_>) -> Result<Instruction<'stat
 
         O::Return => I::Return,
         O::Call { function_index } => I::Call(t.remap(Item::Function, *function_index)?),
-        O::CallIndirect { index, table_index } => I::CallIndirect {
+        O::CallIndirect {
+            index,
+            table_index,
+            table_byte: _,
+        } => I::CallIndirect {
             ty: t.remap(Item::Type, *index)?,
             table: t.remap(Item::Table, *table_index)?,
         },
@@ -898,11 +914,11 @@ pub fn op(t: &mut dyn Translator, op: &Operator<'_>) -> Result<Instruction<'stat
     })
 }
 
-pub fn block_type(t: &mut dyn Translator, ty: &TypeOrFuncType) -> Result<BlockType> {
+pub fn block_type(t: &mut dyn Translator, ty: &wasmparser::BlockType) -> Result<BlockType> {
     match ty {
-        TypeOrFuncType::Type(Type::EmptyBlockType) => Ok(BlockType::Empty),
-        TypeOrFuncType::Type(ty) => Ok(BlockType::Result(t.translate_ty(ty)?)),
-        TypeOrFuncType::FuncType(f) => Ok(BlockType::FunctionType(t.remap(Item::Type, *f)?)),
+        wasmparser::BlockType::Empty => Ok(BlockType::Empty),
+        wasmparser::BlockType::Type(ty) => Ok(BlockType::Result(t.translate_ty(ty)?)),
+        wasmparser::BlockType::FuncType(f) => Ok(BlockType::FunctionType(t.remap(Item::Type, *f)?)),
     }
 }
 
@@ -921,7 +937,7 @@ pub fn data(t: &mut dyn Translator, data: wasmparser::Data<'_>, s: &mut DataSect
             memory_index,
             init_expr,
         } => {
-            offset = t.translate_init_expr(init_expr)?;
+            offset = t.translate_init_expr(init_expr, &Type::I32, InitExprKind::DataOffset)?;
             DataSegmentMode::Active {
                 memory_index: t.remap(Item::Memory, *memory_index)?,
                 offset: &offset,
@@ -947,7 +963,9 @@ pub fn code(t: &mut dyn Translator, body: FunctionBody<'_>, s: &mut CodeSection)
         .collect::<Result<Vec<_>>>()?;
     let mut func = Function::new(locals);
 
-    for op in body.get_operators_reader()? {
+    let mut reader = body.get_operators_reader()?;
+    reader.allow_memarg64(true);
+    for op in reader {
         let op = op?;
         func.instruction(&t.translate_op(&op)?);
     }
