@@ -3,7 +3,7 @@ use wasmparser::Validator;
 
 fn validate(validator: &mut Validator, bytes: &[u8]) {
     let err = match validator.validate_all(bytes) {
-        Ok(()) => return,
+        Ok(_) => return,
         Err(e) => e,
     };
     drop(std::fs::write("test.wasm", &bytes));
